/***************************************************************************
 *cr                                                                       
 *cr            (C) Copyright 1995 The Board of Trustees of the           
 *cr                        University of Illinois                       
 *cr                         All Rights Reserved                        
 *cr                                                                   
 ***************************************************************************/

/***************************************************************************
 * RCS INFORMATION:
 *
 *	$RCSfile: Inform.C,v $
 *	$Author: dalke $	$Locker:  $		$State: Exp $
 *	$Revision: 1.9 $	$Date: 1997/02/07 21:07:11 $
 *
 ***************************************************************************
 * DESCRIPTION:
 *
 * Inform - takes messages and displays them to the given ostream.
 *
 ***************************************************************************/

#include <strstream.h>
#include <string.h>
#include "Inform.h"
#include "utilities.h"

// interface to the error messaging in the GNU C++ class library
#include "builtin.h"
void TBMD_lib_default_error_handler(const char *s1, const char *s2)
{
   cout << "I am going: " << s1 <<endl;
   msgErr << s1 << '-' << s2 << sendmsg;
}
two_arg_error_handler_t lib_error_handler = TBMD_lib_default_error_handler;

// initial 4 instances of this class
Inform msgInfo("Info");
Inform msgWarn("Warning");
Inform msgErr("ERROR");
Inform msgDebug("** Debug **", FALSE);

/////////////////////////  manipulator functions  

// signal we wish to send the message
Inform& sendmsg(Inform& inform) { return inform.send(); }

// set the current msg level
Inform& level1(Inform& inform) { return inform.msg_level(1); }
Inform& level2(Inform& inform) { return inform.msg_level(2); }
Inform& level3(Inform& inform) { return inform.msg_level(3); }
Inform& level4(Inform& inform) { return inform.msg_level(4); }
Inform& level5(Inform& inform) { return inform.msg_level(5); }
Inform& level6(Inform& inform) { return inform.msg_level(6); }
Inform& level7(Inform& inform) { return inform.msg_level(7); }
Inform& level8(Inform& inform) { return inform.msg_level(8); }
Inform& level9(Inform& inform) { return inform.msg_level(9); }
Inform& level10(Inform& inform) { return inform.msg_level(10); }

/////////////////////////  class functions  

// class constructor
Inform::Inform(char *myname, int ison) {
  msgBuf = new char[MAX_MSG_SIZE];
  memset(msgBuf, 0, MAX_MSG_SIZE);
  msg = new ostrstream(msgBuf, MAX_MSG_SIZE);
  
  On = ison;
  msgDest = &cout;
  needNewline = FALSE;
  outputLevel = msgLevel = MIN_MSG_LEVEL;
  
  name = stringdup(myname);
}


// class destructor ... frees up space
Inform::~Inform(void) {
  delete msg;
  delete [] msgBuf;
  delete [] name;
}


// print out the message in the given buffer.  Will modify the string,
// so beware.
void Inform::display_message(char *buf) {
  char *msgptr;

  if(on() && msgLevel <= outputLevel) {
    if(needNewline) {
      (*msgDest) << endl;
      needNewline = FALSE;
    }
    msgptr = strtok(buf, "\n");
    while(msgptr) {
      (*msgDest) << name;
      if(msgLevel > 1)
        (*msgDest) << " " << msgLevel;
      (*msgDest) << ") " << msgptr << endl;
      msgptr = strtok(NULL, "\n");
    }
  }
}


// the signal has been given ... process the message.  Return ref to object.
Inform& Inform::send(void) {
  if(*msgBuf) {
    display_message(msgBuf);
    msgBuf[0] = '\0';
    delete msg;
    memset(msgBuf, 0, MAX_MSG_SIZE);
    msg = new ostrstream(msgBuf, MAX_MSG_SIZE);
    msgLevel = MIN_MSG_LEVEL;		// reset message level
  }
    
  return *this;
}

